package name.dsouflis.aop.tracing;

import org.apache.bcel.Repository;
import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.util.ClassPath;

/*
 * AddTracing.java
 *
 * Created on April 2, 2002, 6:53 PM
 */


/** Utility that instruments a class with the same trace output that
 * {@link TracingClassLoader} uses.&nbsp; The target class is actually
 * modified on disk, which enables one to inspect the instrumentation
 * with <CODE>javap</CODE> or BCEL's <CODE>listclass</CODE>.
 * @author dsouflis
 * @version $Revision: 1.4 $
 */
public class AddTracing extends Object {

    /** Prohibits creation of new AddTracing */
    private AddTracing() {
    }

    /** The application entry point.&nbsp; Accepts a single argument,
     * which is a class name.
     * @param args the command line arguments
     */
    public static void main (String args[]) {
        if(args.length==0) {
            return;
        }
        String className=args[0];
        addTracing(className);
    }

    /** Adds tracing instrumentation to a class, using TracingClassLoader's
     * internal methods directly.
     * @param className The class to instrument
     */
    public static void addTracing(String className) {
        final java.util.Vector classNames=new java.util.Vector(1);
        classNames.add(className);
        ClassMethodFilter flt=new ClassMethodFilter() {
            public boolean classHasAdvice(String className) {
                return classNames.indexOf(className)!=-1;
            }
            public boolean methodHasAdvice(String className, String methodName, String signature) {
                return true;
            }
        };
        JavaClass clazz=TracingClassLoader.classWithTracing(flt,className);
        ClassPath.ClassFile cf= Repository.lookupClassFile(className);
        try {
            clazz.dump(cf.getPath());
        }
        catch(java.io.IOException e) {
            e.printStackTrace();
        }
    }
}