/*
 * ClassMethodFilter.java
 *
 * Created on May 14, 2002, 9:55 PM
 */

package name.dsouflis.aop.tracing;

/** When using TracingClassLoader, client programs must supply an instance of
 * this interface to tell the ClassLoader which classes and methods it should instrument.<br>
 * In any case, TracingClassLoader will refuse to trace classes in java.* packages and constructors.<br>
 * Code added non-invasively is commonly called 'advice' in AOP, hence the names of the methods.
 *
 * @author  dsouflis
 * @version 1.0
 */
public interface ClassMethodFilter {
    /** Return <code>true</code> if this class must be traced.
     * @param className Guess what
     * @return Whether the class must be traced
     */
    public boolean classHasAdvice(String className);
    /** Return <code>true</code> if this method in this class must be traced
     * @param className Guess what
     * @param methodName Guess what
     * @param signature The method's signature (refer to the JLS)
     * @return Whether this method must be traced
     */
    public boolean methodHasAdvice(String className, String methodName, String signature);
}
