package name.dsouflis.aop.tracing;

import java.lang.reflect.*;

/**
 * Runs class using TracingClassLoader, user specifies class to trace.&nbsp;
 * Adapted from BCEL's de.fub.bytecode.util.JavaWrapper.<br>
 * Original credits follow:
 * Java interpreter replacement, i.e., wrapper that uses its own ClassLoader
 * to modify/generate classes as they're requested. You can take this as a template
 * for your own applications.<br>
 * @version $Id: JavaWrapper.java,v 1.3 2001/08/24 13:56:51 dahm Exp $
 * @author  <A HREF="http://www.berlin.de/~markus.dahm/">M. Dahm</A>
 * @see ClassLoader
 */
public class Trace {
    private java.lang.ClassLoader loader;

    public Trace(java.lang.ClassLoader loader) {
        this.loader = loader;
    }

    /** Runs the main method of the given class with the arguments passed in argv
     *
     * @param class_name the fully qualified class name
     * @param argv the arguments just as you would pass them directly
     */
    public void runMain(String class_name, String[] argv) throws ClassNotFoundException
    {
        Class   cl    = loader.loadClass(class_name);
        Method method = null;

        try {
        method = cl.getMethod("main",  new Class[] { argv.getClass() });

            /* Method main is sane ?
             */
            int   m = method.getModifiers();
            Class r = method.getReturnType();

            if(!(Modifier.isPublic(m) && Modifier.isStatic(m)) ||
            Modifier.isAbstract(m) || (r != Void.TYPE))
            throw new NoSuchMethodException();
        } catch(NoSuchMethodException no) {
            System.out.println("In class " + class_name +
            ": public static void main(String[] argv) is not defined");
            return;
        }

        try {
        method.invoke(null, new Object[] { argv });
        } catch(Exception ex) {
            ex.printStackTrace();
        }
    }

    /** Default main method used as wrapper, expects a comma-separated list of 
     * the fully qualified class names
     * of the classes to trace as the first argument and the real class to run as a second one.
     */
    public static void main(String[] argv) throws Exception {
        /* Expects class name to trace as first argument, class name to run as second argument,
         *other arguments are passed through.
         */
        if(argv.length <2) {
            System.out.println("Missing class name to trace and class name to run.");
            return;
        }

        String classNameToTrace= argv[0];
        String className = argv[1];
        String[] new_argv = new String[argv.length - 2];
        System.arraycopy(argv, 2, new_argv, 0, new_argv.length);

        final java.util.Vector classNamesVec=new java.util.Vector();
        java.util.StringTokenizer tz=new java.util.StringTokenizer(classNameToTrace,",");
        while(tz.hasMoreTokens()) {
            String classNameToken=tz.nextToken();
            classNamesVec.add(classNameToken);
            System.out.print("Will trace ");
            System.out.println(classNameToken);
        }
        ClassMethodFilter flt=new ClassMethodFilter() {
            public boolean classHasAdvice(String className) {
                boolean outcome=classNamesVec.indexOf(className)!=-1;
                if(outcome) {
                    System.out.print("Tracing ");
                    System.out.println(className);
                }
                return outcome;
            }
            public boolean methodHasAdvice(String className, String methodName, String signature) {
                return true;
            }
        };
        ClassLoader loader=new TracingClassLoader(flt);
        Trace wrapper = new Trace(loader);
        wrapper.runMain(className, new_argv);
    }
}

